package sun.rmi.registry;

import java.security.ProtectionDomain;
import java.io.FilePermission;
import java.security.Permissions;
import java.security.Policy;
import java.security.CodeSource;
import java.security.cert.Certificate;
import java.security.PrivilegedAction;
import java.security.PermissionCollection;
import java.security.AccessControlContext;
import java.text.MessageFormat;
import sun.rmi.server.LoaderHandler;
import java.net.URLClassLoader;
import sun.misc.URLClassPath;
import java.rmi.RMISecurityManager;
import java.rmi.server.UID;
import java.rmi.activation.ActivationID;
import sun.rmi.server.UnicastRef;
import java.lang.reflect.Proxy;
import java.util.MissingResourceException;
import java.rmi.server.ServerNotActiveException;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.rmi.AccessException;
import java.rmi.AlreadyBoundException;
import java.rmi.NotBoundException;
import sun.rmi.server.UnicastServerRef;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.net.SocketPermission;
import java.security.Permission;
import java.security.PrivilegedExceptionAction;
import sun.rmi.server.UnicastServerRef2;
import sun.rmi.transport.LiveRef;
import java.rmi.RemoteException;
import java.rmi.server.RMIServerSocketFactory;
import java.rmi.server.RMIClientSocketFactory;
import sun.rmi.runtime.Log;
import java.security.Security;
import sun.misc.ObjectInputFilter;
import java.util.ResourceBundle;
import java.rmi.server.ObjID;
import java.net.InetAddress;
import java.rmi.Remote;
import java.util.Hashtable;
import java.rmi.registry.Registry;
import java.rmi.server.RemoteServer;

public class RegistryImpl extends RemoteServer implements Registry
{
    private static final long serialVersionUID = 4666870661827494597L;
    private Hashtable<String, Remote> bindings;
    private static Hashtable<InetAddress, InetAddress> allowedAccessCache;
    private static RegistryImpl registry;
    private static ObjID id;
    private static ResourceBundle resources;
    private static final String REGISTRY_FILTER_PROPNAME = "sun.rmi.registry.registryFilter";
    private static final int REGISTRY_MAX_DEPTH = 20;
    private static final int REGISTRY_MAX_ARRAY_SIZE = 1000000;
    private static final ObjectInputFilter registryFilter;
    
    private static ObjectInputFilter initRegistryFilter() {
        ObjectInputFilter filter2 = null;
        String s = System.getProperty("sun.rmi.registry.registryFilter");
        if (s == null) {
            s = Security.getProperty("sun.rmi.registry.registryFilter");
        }
        if (s != null) {
            filter2 = ObjectInputFilter.Config.createFilter2(s);
            Log log = Log.getLog("sun.rmi.registry", "registry", -1);
            if (log.isLoggable(Log.BRIEF)) {
                log.log(Log.BRIEF, "registryFilter = " + filter2);
            }
        }
        return filter2;
    }
    
    public RegistryImpl(int n, RMIClientSocketFactory rmiClientSocketFactory, RMIServerSocketFactory rmiServerSocketFactory) throws RemoteException {
        this(n, rmiClientSocketFactory, rmiServerSocketFactory, (sun.misc.ObjectInputFilter<sun.rmi.registry.RegistryImpl,sun.misc.ObjectInputFilter.Status>)RegistryImpl::registryFilter);
    }
    
    public RegistryImpl(final int n, final RMIClientSocketFactory csf, final RMIServerSocketFactory ssf, final ObjectInputFilter objectInputFilter) throws RemoteException {
        this.bindings = new Hashtable(101);
        if (n == 1099 && System.getSecurityManager() != null) {
            try {
                AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                    @Override
                    public Void run() throws RemoteException {
                        RegistryImpl.access$100(RegistryImpl.this, new UnicastServerRef2(new LiveRef(RegistryImpl.id, n, csf, ssf), objectInputFilter));
                        return null;
                    }
                }, null, new Permission[] { new SocketPermission("localhost:" + n, "listen,accept") });
            }
            catch (PrivilegedActionException ex) {
                throw (RemoteException)ex.getException();
            }
        }
        else {
            this.setup(new UnicastServerRef2(new LiveRef(RegistryImpl.id, n, csf, ssf), objectInputFilter));
        }
    }
    
    public RegistryImpl(final int n) throws RemoteException {
        this.bindings = new Hashtable(101);
        if (n == 1099 && System.getSecurityManager() != null) {
            try {
                AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                    @Override
                    public Void run() throws RemoteException {
                        RegistryImpl.access$100(RegistryImpl.this, new UnicastServerRef(new LiveRef(RegistryImpl.id, n), filterInfo/* sun.misc.ObjectInputFilter.FilterInfo, */ -> RegistryImpl.access$200(filterInfo)));
                        return null;
                    }
                }, null, new Permission[] { new SocketPermission("localhost:" + n, "listen,accept") });
            }
            catch (PrivilegedActionException ex) {
                throw (RemoteException)ex.getException();
            }
        }
        else {
            this.setup(new UnicastServerRef(new LiveRef(RegistryImpl.id, n), (sun.misc.ObjectInputFilter<sun.rmi.registry.RegistryImpl,sun.misc.ObjectInputFilter.Status>)RegistryImpl::registryFilter));
        }
    }
    
    private void setup(UnicastServerRef ref) throws RemoteException {
        (this.ref = ref).exportObject(this, null, true);
    }
    
    @Override
    public Remote lookup(String s) throws RemoteException, NotBoundException {
        synchronized (this.bindings) {
            Remote remote = (Remote)this.bindings.get(s);
            if (remote == null) {
                throw new NotBoundException(s);
            }
            return remote;
        }
    }
    
    @Override
    public void bind(String s, Remote value) throws RemoteException, AlreadyBoundException, AccessException {
        synchronized (this.bindings) {
            if ((Remote)this.bindings.get(s) != null) {
                throw new AlreadyBoundException(s);
            }
            this.bindings.put(s, value);
        }
    }
    
    @Override
    public void unbind(String s) throws RemoteException, NotBoundException, AccessException {
        synchronized (this.bindings) {
            if ((Remote)this.bindings.get(s) == null) {
                throw new NotBoundException(s);
            }
            this.bindings.remove(s);
        }
    }
    
    @Override
    public void rebind(String key, Remote value) throws RemoteException, AccessException {
        this.bindings.put(key, value);
    }
    
    @Override
    public String[] list() throws RemoteException {
        String[] array;
        synchronized (this.bindings) {
            int size = this.bindings.size();
            array = new String[size];
            Enumeration<String> keys = this.bindings.keys();
            while (--size >= 0) {
                array[size] = (String)keys.nextElement();
            }
        }
        return array;
    }
    
    public static void checkAccess(String str) throws AccessException {
        try {
            final String clientHost = RemoteServer.getClientHost();
            final InetAddress inetAddress;
            try {
                inetAddress = (InetAddress)AccessController.doPrivileged(new PrivilegedExceptionAction<InetAddress>() {
                    @Override
                    public InetAddress run() throws UnknownHostException {
                        return InetAddress.getByName(clientHost);
                    }
                });
            }
            catch (PrivilegedActionException ex) {
                throw (UnknownHostException)ex.getException();
            }
            if (RegistryImpl.allowedAccessCache.get(inetAddress) == null) {
                if (inetAddress.isAnyLocalAddress()) {
                    throw new AccessException(str + " disallowed; origin unknown");
                }
                try {
                    AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                        @Override
                        public Void run() throws IOException {
                            new ServerSocket(0, 10, inetAddress).close();
                            RegistryImpl.allowedAccessCache.put(inetAddress, inetAddress);
                            return null;
                        }
                    });
                }
                catch (PrivilegedActionException ex2) {
                    throw new AccessException(str + " disallowed; origin " + inetAddress + " is non-local host");
                }
            }
        }
        catch (ServerNotActiveException ex3) {}
        catch (UnknownHostException ex4) {
            throw new AccessException(str + " disallowed; origin is unknown host");
        }
    }
    
    public static ObjID getID() {
        return RegistryImpl.id;
    }
    
    private static String getTextResource(String str) {
        if (RegistryImpl.resources == null) {
            try {
                RegistryImpl.resources = ResourceBundle.getBundle("sun.rmi.registry.resources.rmiregistry");
            }
            catch (MissingResourceException ex) {}
            if (RegistryImpl.resources == null) {
                return "[missing resource file: " + str + "]";
            }
        }
        String string = null;
        try {
            string = RegistryImpl.resources.getString(str);
        }
        catch (MissingResourceException ex2) {}
        if (string == null) {
            return "[missing resource: " + str + "]";
        }
        return string;
    }
    
    private static ObjectInputFilter.Status registryFilter(ObjectInputFilter.FilterInfo filterInfo) {
        if (RegistryImpl.registryFilter != null) {
            ObjectInputFilter.Status checkInput = RegistryImpl.registryFilter.checkInput(filterInfo);
            if (checkInput != ObjectInputFilter.Status.UNDECIDED) {
                return checkInput;
            }
        }
        if (filterInfo.depth() > 20L) {
            return ObjectInputFilter.Status.REJECTED;
        }
        Class<Object> serialClass = filterInfo.serialClass();
        if (serialClass != null) {
            if (serialClass.isArray()) {
                return (filterInfo.arrayLength() >= 0L && filterInfo.arrayLength() > 1000000L) ? ObjectInputFilter.Status.REJECTED : ObjectInputFilter.Status.UNDECIDED;
            }
            if (String.class == serialClass || Number.class.isAssignableFrom(serialClass) || Remote.class.isAssignableFrom(serialClass) || Proxy.class.isAssignableFrom(serialClass) || UnicastRef.class.isAssignableFrom(serialClass) || RMIClientSocketFactory.class.isAssignableFrom(serialClass) || RMIServerSocketFactory.class.isAssignableFrom(serialClass) || ActivationID.class.isAssignableFrom(serialClass) || UID.class.isAssignableFrom(serialClass)) {
                return ObjectInputFilter.Status.ALLOWED;
            }
            return ObjectInputFilter.Status.REJECTED;
        }
        else {
            return ObjectInputFilter.Status.UNDECIDED;
        }
    }
    
    public static void main(String[] array) {
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }
        try {
            String property = System.getProperty("env.class.path");
            if (property == null) {
                property = ".";
            }
            URLClassLoader o = new URLClassLoader(URLClassPath.pathToURLs(property));
            LoaderHandler.registerCodebaseLoader(o);
            Thread.currentThread().setContextClassLoader(o);
            final int n = (array.length >= 1) ? Integer.parseInt(array[0]) : 1099;
            try {
                RegistryImpl.registry = (RegistryImpl)AccessController.doPrivileged(new PrivilegedExceptionAction<RegistryImpl>() {
                    @Override
                    public RegistryImpl run() throws RemoteException {
                        return new RegistryImpl(n);
                    }
                }, getAccessControlContext(n));
            }
            catch (PrivilegedActionException ex) {
                throw (RemoteException)ex.getException();
            }
            while (true) {
                while (true) {
                    try {
                        Thread.sleep(Long.MAX_VALUE);
                        continue;
                    }
                    catch (InterruptedException ex2) {}
                }
            }
        }
        catch (NumberFormatException ex3) {
            System.err.println(MessageFormat.format(getTextResource("rmiregistry.port.badnumber"), new Object[] { array[0] }));
            System.err.println(MessageFormat.format(getTextResource("rmiregistry.usage"), new Object[] { "rmiregistry" }));
        }
        catch (Exception ex4) {
            ex4.printStackTrace();
        }
        System.exit(1);
    }
    
    private static AccessControlContext getAccessControlContext(int i) {
        PermissionCollection permissions = (PermissionCollection)AccessController.doPrivileged(new PrivilegedAction<PermissionCollection>() {
            @Override
            public PermissionCollection run() {
                CodeSource codesource = new CodeSource(null, (Certificate[])null);
                Policy policy = Policy.getPolicy();
                if (policy != null) {
                    return policy.getPermissions(codesource);
                }
                return new Permissions();
            }
        });
        permissions.add(new SocketPermission("*", "connect,accept"));
        permissions.add(new SocketPermission("localhost:" + i, "listen,accept"));
        permissions.add(new RuntimePermission("accessClassInPackage.sun.jvmstat.*"));
        permissions.add(new RuntimePermission("accessClassInPackage.sun.jvm.hotspot.*"));
        permissions.add(new FilePermission("<<ALL FILES>>", "read"));
        return new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(new CodeSource(null, (Certificate[])null), permissions) });
    }
    
    static void access$100(RegistryImpl registryImpl, UnicastServerRef unicastServerRef) throws RemoteException {
        registryImpl.setup(unicastServerRef);
    }
    
    static ObjectInputFilter.Status access$200(ObjectInputFilter.FilterInfo filterInfo) {
        return registryFilter(filterInfo);
    }
    
    static {
        RegistryImpl.allowedAccessCache = new Hashtable(3);
        RegistryImpl.id = new ObjID(0);
        RegistryImpl.resources = null;
        registryFilter = (ObjectInputFilter)AccessController.doPrivileged(RegistryImpl::initRegistryFilter);
    }
}
